"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.expressionCompiler = exports.compileFlags = void 0;
const nullPart_1 = require("./parts/nullPart");
const selfPart_1 = require("./parts/selfPart");
const stack_1 = require("./../utils/stack");
const StringBuilder_1 = require("typescript-dotnet-commonjs/System/Text/StringBuilder");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const operationPart_1 = require("./parts/operationPart");
const partList_1 = require("./parts/partList");
const indicatorPart_1 = require("./parts/indicatorPart");
const literalPart_1 = require("./parts/literalPart");
const propertyPart_1 = require("./parts/propertyPart");
const bracketPart_1 = require("./parts/bracketPart");
const indexerPart_1 = require("./parts/indexerPart");
const functionCallPart_1 = require("./parts/functionCallPart");
const switchCasePart_1 = require("./parts/switchCasePart");
const aggregateFunctionCallPart_1 = require("./parts/aggregateFunctionCallPart");
const utils_1 = require("./utils");
const unterminatedStringLiteralException_1 = require("../error/unterminatedStringLiteralException");
const unterminatedCharacterLiteralException_1 = require("../error/unterminatedCharacterLiteralException");
const unknownFunctionException_1 = require("../error/unknownFunctionException");
const badNumericLiteralException_1 = require("../error/badNumericLiteralException");
const badCharacterLiteralException_1 = require("../error/badCharacterLiteralException");
const badExpressionException_1 = require("../error/badExpressionException");
const functionsList_1 = require("../functions/functionsList");
const operatorsList_1 = require("../operators/operatorsList");
const outputVariables_1 = require("../utils/outputVariables");
const general_1 = require("../utils/general");
class compileFlags {
    constructor(original) {
        this._legacyMode = false;
        this._aggregationSection = false;
        this._bracketPart = false;
        this._indexerPart = false;
        this._switchCasePart = false;
        if (original != undefined) {
            this.legacyMode = original.legacyMode;
            this.aggregationSection = original.aggregationSection;
            this.bracketPart = original.bracketPart;
            this.indexerPart = original.indexerPart;
            this.switchCasePart = original.switchCasePart;
        }
    }
    get legacyMode() {
        return this._legacyMode;
    }
    set legacyMode(value) {
        this._legacyMode = value;
    }
    get aggregationSection() {
        return this._aggregationSection;
    }
    set aggregationSection(value) {
        this._aggregationSection = value;
    }
    get bracketPart() {
        return this._bracketPart;
    }
    set bracketPart(value) {
        this._bracketPart = value;
    }
    get indexerPart() {
        return this._indexerPart;
    }
    set indexerPart(value) {
        this._indexerPart = value;
    }
    get switchCasePart() {
        return this._switchCasePart;
    }
    set switchCasePart(value) {
        this._switchCasePart = value;
    }
}
exports.compileFlags = compileFlags;
class expressionCompiler {
    static ProcessLegacy(expression) {
        expression = expression.replace('Format\(([^""][^\)]+[^""])\)', 'Format("{$1}")');
        return expression;
    }
    static compile(path, startIndex, endIndex, flags) {
        var _a, _b;
        if (startIndex === undefined) {
            if (path == null) {
                return undefined;
            }
            return expressionCompiler.compile(path, 0, (path.length - 1), flags);
        }
        else {
            if (path.trim() === ".") {
                return new selfPart_1.selfPart();
            }
            let sbPart = new StringBuilder_1.StringBuilder();
            let currentPart = undefined;
            let postFixList = new List_1.default();
            let operationsStack = new stack_1.Stack();
            let i = startIndex;
            let isNumber = false;
            let isFloatingPoint = false;
            let newExpression = true;
            let inString = false;
            let inChar = false;
            let isDot = false;
            let isIndicator = false;
            let escapeNext = false;
            let stringChar = "\"";
            let stringCharSingleQuoat = "\'";
            let StartNewPart = (isNewExpression) => {
                newExpression = isNewExpression ? true : newExpression;
                isNumber = false;
                isFloatingPoint = false;
                isDot = false;
            };
            let AddNewPart = (nextPart) => {
                if (nextPart == undefined)
                    return;
                if (currentPart instanceof partList_1.partList) {
                    currentPart.addSubPart(nextPart);
                }
                else if (currentPart == undefined) {
                    currentPart = nextPart;
                }
                else {
                    let ep = new partList_1.partList();
                    ep.addSubPart(currentPart);
                    ep.addSubPart(nextPart);
                    currentPart = ep;
                }
            };
            let MarkEnd = (final) => {
                let nextPart = undefined;
                let val = sbPart.toString() == "\"" ? "" : sbPart.toString();
                sbPart.clear();
                if (isIndicator) {
                    nextPart = new indicatorPart_1.indicatorPart();
                    nextPart.value = val;
                    isIndicator = false;
                }
                else if (inString) {
                    nextPart = new literalPart_1.literalPart();
                    nextPart.value = val;
                }
                else if (val.length > 0) {
                    if (inChar) {
                        nextPart = new literalPart_1.literalPart();
                        nextPart.value = val[0];
                    }
                    else if (val.equals("true") || val.equals("false")) {
                        nextPart = new literalPart_1.literalPart();
                        nextPart.value = val.equals("true");
                    }
                    else if (val.equals("null")) {
                        nextPart = new nullPart_1.nullPart();
                    }
                    else if (isNumber) {
                        try {
                            if (val.equals(".")) {
                                nextPart = new selfPart_1.selfPart();
                            }
                            else {
                                let numeric = 0;
                                if (val.endsWith("f") || val.endsWith("d")) {
                                    isFloatingPoint = true;
                                }
                                if (isFloatingPoint) {
                                    if (val.endsWith("f") || val.endsWith("F"))
                                        numeric = Number.parseFloat(val.substring(0, val.length - 1));
                                    else if (val.endsWith("d") || val.endsWith("D"))
                                        numeric = Number.parseFloat(val.substring(0, val.length - 1));
                                    else
                                        numeric = Number.parseFloat(val);
                                }
                                else {
                                    if (val.endsWith("l") || val.endsWith("L"))
                                        numeric = Number.parseInt(val.substring(0, val.length - 1));
                                    else
                                        numeric = Number.parseInt(val);
                                }
                                nextPart = new literalPart_1.literalPart();
                                nextPart.value = numeric;
                            }
                        }
                        catch (ex) {
                            throw new badNumericLiteralException_1.badNumericLiteralException(val);
                        }
                        // catch (OverflowException ex)
                        // {
                        //     throw new BadNumericLiteralException(val, ex);
                        // }
                    }
                    else {
                        nextPart = new propertyPart_1.propertyPart();
                        nextPart.propertyName = val;
                    }
                }
                AddNewPart(nextPart);
                if (final && currentPart != null) {
                    postFixList.add(currentPart);
                    currentPart = undefined;
                }
                isDot = false;
            };
            for (let i = startIndex; (i <= endIndex); i++) {
                let c = path[i];
                let currentOperator = null;
                if (flags.switchCasePart && newExpression && c == "#") {
                    isIndicator = true;
                }
                else if (((c == stringChar && !inChar) || (flags.legacyMode && c == '\'' && (!inString || stringChar == '\''))) && !escapeNext) {
                    newExpression = false;
                    isNumber = false;
                    isFloatingPoint = false;
                    if (c == '\'')
                        stringChar = '\'';
                    if (inString) {
                        MarkEnd(false);
                        StartNewPart(false);
                        stringChar = '"';
                    }
                    inString = !inString;
                }
                else if (inString && !inChar) {
                    newExpression = false;
                    if (c == '\\' && !escapeNext) {
                        escapeNext = true;
                    }
                    else {
                        sbPart.append(c);
                        escapeNext = false;
                    }
                }
                else if (c == '\'' && !escapeNext && !inString) {
                    newExpression = false;
                    isNumber = false;
                    isFloatingPoint = false;
                    if (inChar) {
                        MarkEnd(false);
                        StartNewPart(false);
                    }
                    inChar = !inChar;
                }
                else if ((inChar && !inString)) {
                    newExpression = false;
                    if (c == '\\' && !escapeNext) {
                        escapeNext = true;
                    }
                    else {
                        if ((sbPart.toString().length < 1)) {
                            sbPart.append(c);
                        }
                        else {
                            throw new badCharacterLiteralException_1.badCharacterLiteralException(sbPart.toString());
                        }
                        escapeNext = false;
                    }
                }
                else if (!inString && !inChar) {
                    if (String.isWhiteSpace(c)) {
                        if ((sbPart.toString().length > 0)) {
                            MarkEnd(true);
                        }
                        StartNewPart(false);
                        continue;
                    }
                    else if (newExpression && !isDot && (general_1.isNumberType(c) || c == '-' || c == '.')) {
                        newExpression = false;
                        isNumber = true;
                        isFloatingPoint = c == '.';
                        sbPart.append(c);
                    }
                    else if (isNumber && !isDot && (general_1.isNumberType(c) || (c == "." && !isFloatingPoint))) {
                        if ((c == '.')) {
                            isFloatingPoint = true;
                        }
                        sbPart.append(c);
                    }
                    else if (operatorsList_1.operatorsList.binaryOperators.containsKey(c + "") && !String.isNullOrEmpty(currentOperator = c + "")) {
                        MarkEnd(true);
                        if ((flags.legacyMode
                            && (currentOperator == "%"))) {
                            currentOperator = "#";
                        }
                        let op = new operationPart_1.operationPart();
                        op.operator = operatorsList_1.operatorsList.binaryOperators.getValue(currentOperator);
                        while (((operationsStack.count() > 0) && (((_a = operationsStack.peek().operator) === null || _a === void 0 ? void 0 : _a.precedence) >= ((_b = op.operator) === null || _b === void 0 ? void 0 : _b.precedence)))) {
                            let peekPart = operationsStack.pop();
                            postFixList.add(peekPart);
                        }
                        operationsStack.push(op);
                        StartNewPart(true);
                    }
                    else if ((c == '{')) {
                        MarkEnd(false);
                        let subEndIndex = new outputVariables_1.outputVariables();
                        subEndIndex.endIndex = 0;
                        let switchSubject = currentPart;
                        currentPart = undefined;
                        AddNewPart(expressionCompiler.parseSwitchCasePart(path, i, /* out */ subEndIndex, switchSubject, flags));
                        MarkEnd(false);
                        StartNewPart(false);
                        i = subEndIndex.endIndex;
                        newExpression = false;
                    }
                    else if ((c == '(')) {
                        let subEndIndex = new outputVariables_1.outputVariables();
                        if ((sbPart.toString().length > 0)) {
                            let ep = null;
                            ep = expressionCompiler.parseFunctionCallPart(path, i, /* out */ subEndIndex, sbPart.toString(), flags);
                            sbPart.clear();
                            AddNewPart(ep);
                        }
                        else if (newExpression) {
                            AddNewPart(expressionCompiler.parseBracketPart(path, i, /* out */ subEndIndex, flags));
                        }
                        else {
                            throw new badExpressionException_1.badExpressionException("Found '(' where expected property");
                        }
                        i = subEndIndex.endIndex;
                        newExpression = false;
                    }
                    else if ((c == '[')) {
                        newExpression = false;
                        if ((sbPart.toString().length > 0)) {
                            MarkEnd(false);
                        }
                        // let subEndIndex: number = 0;
                        let subEndIndex = new outputVariables_1.outputVariables();
                        subEndIndex.endIndex = 0;
                        AddNewPart(expressionCompiler.parseIndexerPart(path, i, subEndIndex, flags));
                        StartNewPart(false);
                        i = subEndIndex.endIndex;
                    }
                    else if (((c == '.')
                        && (!isNumber
                            && !isFloatingPoint))) {
                        if ((sbPart.toString().length > 0)) {
                            MarkEnd(false);
                        }
                        isDot = true;
                        newExpression = false;
                    }
                    else {
                        sbPart.append(c);
                        newExpression = false;
                    }
                }
            }
            if (inString) {
                throw new unterminatedStringLiteralException_1.unterminatedStringLiteralException();
            }
            else if (inChar) {
                throw new unterminatedCharacterLiteralException_1.unterminatedCharacterLiteralException();
            }
            MarkEnd(true);
            while ((operationsStack.count() > 0)) {
                postFixList.add(operationsStack.pop());
            }
            let result = expressionCompiler.solvePostFix(postFixList);
            if ((result instanceof propertyPart_1.propertyPart && (flags.indexerPart && flags.legacyMode))) {
                let oldResult = result.propertyName;
                result = new literalPart_1.literalPart();
                result.value = oldResult;
            }
            if (flags.bracketPart) {
                let bp = new bracketPart_1.bracketPart();
                bp.addSubPart(result);
                return bp;
            }
            else {
                return result;
            }
        }
    }
    static solvePostFix(postFixList) {
        let partStack = new stack_1.Stack();
        postFixList.forEach((item) => {
            if (!(item instanceof operationPart_1.operationPart)) {
                partStack.push(item);
            }
            else {
                let op = item;
                op.parameter2 = partStack.pop();
                op.parameter1 = partStack.pop();
                partStack.push(op);
            }
        });
        return partStack.peek();
    }
    static parseBracketPart(path, startIndex, /* out */ endIndex, flags) {
        endIndex.endIndex = startIndex;
        utils_1.utils.markEnclosure(path, startIndex, "(", ")", /* out */ endIndex);
        let compileFlag = new compileFlags(flags);
        compileFlag.bracketPart = true;
        let bp = (expressionCompiler.compile(path, (startIndex + 1), (endIndex.endIndex - 1), compileFlag));
        bp.BracketEnded = true;
        return bp;
    }
    static parseFunctionCallPart(path, startIndex, /* out */ endIndex, functionName, flags) {
        let parameterTokens = utils_1.utils.splitEnclosure(path, startIndex, "(", ")", /* out */ endIndex, ",");
        if (functionsList_1.functionsList.ExpressionFunctions.containsKey(functionName)) {
            let functionCallPart1 = new functionCallPart_1.functionCallPart();
            functionCallPart1.function = functionsList_1.functionsList.ExpressionFunctions.getValue(functionName);
            functionCallPart1.parameters = functionCallPart1.function.compileParameters(parameterTokens, flags);
            return functionCallPart1;
        }
        else if ((flags.aggregationSection && functionsList_1.functionsList.AggregateFunctions.containsKey(functionName))) {
            let functionCallPart = new aggregateFunctionCallPart_1.aggregateFunctionCallPart();
            functionCallPart.Function = functionsList_1.functionsList.AggregateFunctions.getValue(functionName);
            functionCallPart.parameters = functionCallPart.Function.compileParameters(parameterTokens, flags);
            return functionCallPart;
        }
        else {
            throw new unknownFunctionException_1.unknownFunctionException(functionName.toString());
        }
    }
    static parseIndexerPart(path, startIndex, endIndex, flags) {
        let result = new indexerPart_1.indexerPart();
        utils_1.utils.markEnclosure(path, startIndex, "[", "]", /* out */ endIndex);
        const indexCompileFlag = new compileFlags(flags);
        indexCompileFlag.indexerPart = true;
        result.indexerExpression = expressionCompiler.compile(path, (startIndex + 1), (endIndex.endIndex - 1), indexCompileFlag);
        return result;
    }
    static parseSwitchCasePart(path, startIndex, endIndex, expression, flags) {
        let scp = new switchCasePart_1.switchCasePart();
        scp.expression = expression;
        scp.switchData = utils_1.utils.buildSwitchCase(path, startIndex, /* out */ endIndex, new compileFlags(flags));
        return scp;
    }
}
exports.expressionCompiler = expressionCompiler;
//# sourceMappingURL=expressionCompiler.js.map