"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.propertyPart = void 0;
const expressionPart_1 = require("./expressionPart");
const utils_1 = require("../utils");
const stack_1 = require("../../utils/stack");
const reflectionExtender_1 = require("../../utils/reflectionExtender");
const dynamicObject_1 = require("../../utils/dynamicObject");
const selection_1 = require("../../functions/selection");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const legacyBuiltInProperty_1 = require("../../utils/legacyBuiltInProperty");
const entityObject_1 = require("../../utils/entityObject");
const dictionaryObject_1 = require("../../utils/dictionaryObject");
const general_1 = require("../../utils/general");
class propertyPart extends expressionPart_1.expressionPart {
    constructor() {
        super(...arguments);
        this.propertyName = "";
    }
    get simpleName() {
        return this.propertyName;
    }
    resolve(expressionDataobj, resolved, running, evaluateAllContexts = true, setValue = undefined) {
        var _a;
        try {
            if (expressionDataobj != null) {
                let parentData;
                parentData = (_a = expressionDataobj.parent) !== null && _a !== void 0 ? _a : expressionDataobj;
                let tempContext = expressionDataobj.contexts.clone();
                while (!tempContext.isEmpty()) {
                    let context = tempContext.pop();
                    if (context instanceof stack_1.Stack) {
                        context = context.pop();
                    }
                    if (!String.isNullOrEmpty(this.propertyName)) {
                        const prop = general_1.getDirectProperty(context, this.propertyName);
                        if (prop != null) {
                            resolved.resolved = true;
                            return prop;
                        }
                        if (context instanceof List_1.default) {
                            if (Object.keys(legacyBuiltInProperty_1.default).includes(this.propertyName.toLowerCase())) {
                                const innerResult = general_1.getDirectProperty(context, this.propertyName.toLowerCase());
                                if (innerResult !== undefined) {
                                    resolved.resolved = true;
                                    return innerResult;
                                }
                            }
                        }
                        if (context instanceof entityObject_1.EntityObject) {
                            let castedSubject = (context);
                            const result = castedSubject.getItem(this.propertyName);
                            if (result !== undefined) {
                                resolved.resolved = (result != undefined);
                                return result;
                            }
                        }
                        if (this.property == null || this.contextType != reflectionExtender_1.getType(context)) {
                            this.property = utils_1.utils.getProperty(reflectionExtender_1.getType(context), this.propertyName);
                            this.contextType = reflectionExtender_1.getType(context);
                        }
                        if (this.property != null) {
                            if (setValue != null)
                                this.property.setValue(context, setValue);
                            resolved.resolved = true;
                            const result = this.property.getValue(context);
                            // parentData.resultCache.setValue(cacheKey, result)
                            return result;
                        }
                        else if (context instanceof dictionaryObject_1.DictionaryObject) {
                            var obj = context;
                            if (obj != null) {
                                if (setValue != null)
                                    obj.setValue(this.propertyName, setValue);
                                if (obj.containsKey(this.propertyName)) {
                                    resolved.resolved = true;
                                    // parentData.resultCache.setValue(cacheKey, obj.getValue(this.propertyName));
                                    return this.propertyName;
                                }
                            }
                        }
                        else if (context instanceof dynamicObject_1.dynamicObject) {
                            let obj = context;
                            var memberNames = obj.getDynamicMemberNames();
                            if (obj != null) {
                                if (setValue != null) {
                                    resolved.resolved = true;
                                    general_1.setDirectProperty(obj, this.propertyName, setValue);
                                }
                                else if (memberNames != null && memberNames.contains(this.propertyName)) {
                                    resolved.resolved = true;
                                    let memberObj = obj.getItem(this.propertyName);
                                    // parentData.resultCache.setValue(cacheKey, memberObj);
                                    return memberObj;
                                }
                            }
                        }
                        else if (context instanceof selection_1.selection) {
                            for (let sel of context.toArray()) {
                                let obj = sel;
                                var memberNames = obj.getDynamicMemberNames();
                                if (obj != null) {
                                    if (setValue != null) {
                                        resolved.resolved = true;
                                        general_1.setDirectProperty(obj, this.propertyName, setValue);
                                    }
                                    else if (memberNames != null && memberNames.contains(this.propertyName)) {
                                        resolved.resolved = true;
                                        let memberObj = obj.getItem(this.propertyName);
                                        if (memberObj != undefined) {
                                            // parentData.resultCache.setValue(cacheKey, memberObj);
                                            return memberObj;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (!evaluateAllContexts)
                        break;
                }
            }
        }
        catch (ex) {
            throw ex;
        }
        resolved.resolved = false;
        return undefined;
    }
    toString() {
        return this.propertyName;
    }
}
exports.propertyPart = propertyPart;
//# sourceMappingURL=propertyPart.js.map