"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const stack_1 = require("./utils/stack");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const indexerPart_1 = require("./compiler/parts/indexerPart");
const eventHandlers_1 = require("./utils/events/eventHandlers");
const INotifyPropertyChanged_1 = require("./utils/events/INotifyPropertyChanged");
const expressionCompiler_1 = require("./compiler/expressionCompiler");
const propertyPart_1 = require("./compiler/parts/propertyPart");
const INotifyCollectionChanged_1 = require("./utils/events/INotifyCollectionChanged");
const Utility_1 = require("typescript-dotnet-commonjs/System/Text/Utility");
require("./utils/stringUtil");
class expressionData {
    constructor(expression, pageContext, globalContext, legacyMode = true, convertType = true) {
        this.boundCollections = new List_1.default();
        this.boundObjects = new List_1.default();
        this.subExpressions = new List_1.default();
        this.contexts = new stack_1.Stack();
        this.convertType = false;
        this.monitorExpression = false;
        this.cacheEnabled = false;
        this.legacyMode = false;
        this.isAlreadyLegacy = false;
        this.constructorInit(expression, pageContext, globalContext, legacyMode, convertType, false);
        this.cacheEnabled = true;
    }
    OnControlDestroyed(control) {
        if ((this.controlDestroyed != null)) {
            this.controlDestroyed.invoke(null, control);
        }
    }
    constructorInit(expression, pageContext, globalContext, legacyMode, convertType = true, isAlreadyLegacy = false) {
        this.expression = expression;
        this.convertType = convertType;
        this.globalContext = this.globalContext;
        this.pageContext = this.pageContext;
        this.isAlreadyLegacy = isAlreadyLegacy;
        this.legacyMode = legacyMode;
        this.cacheEnabled = true;
        this.contexts = new stack_1.Stack();
        if ((this.globalContext != null)) {
            this.contexts.push(this.globalContext);
        }
        if ((this.pageContext != null)) {
            this.contexts.push(this.pageContext);
        }
        this.toLegacyIfNeeded();
        // ControlDestroyed += ExpressionData_ControlDestroyed;
    }
    privateConstructor(expression, contexts, pageContext, globalContext, legacyMode, convertType = true, isAlreadyLegacy = false) {
        this.expression = expression;
        this.convertType = convertType;
        this.globalContext = globalContext;
        this.pageContext = pageContext;
        this.contexts = contexts.clone();
        //this.contexts.push(contexts);
        this.isAlreadyLegacy = isAlreadyLegacy;
        this.legacyMode = legacyMode;
        this.cacheEnabled = true;
        this.toLegacyIfNeeded();
        return this;
        // ControlDestroyed += ExpressionData_ControlDestroyed;
    }
    ExpressionData_ControlDestroyed(sender, boundObject) {
        let removables = new List_1.default();
        this.boundCollections.forEach((item) => {
            if (item.checkBound(boundObject)) {
                removables.add(item);
                item.unbind();
            }
        });
        this.boundObjects.forEach((item) => {
            if (item.checkBound(boundObject)) {
                removables.add(item);
                item.unbind();
            }
        });
        removables.forEach((item) => {
            this.boundCollections.remove(item);
            this.boundObjects.remove(item);
        });
    }
    toSubEx(currentContext = null) {
        return this.toSubExInit(this.expression, currentContext);
    }
    toSubExInit(newExpression, currentContext = null) {
        let subExpressionData = this.privateConstructor(newExpression, this.contexts, this.pageContext, this.globalContext, this.legacyMode, this.convertType, this.isAlreadyLegacy);
        if (((currentContext != null)
            && ((currentContext != this.pageContext)
                && ((currentContext != this.globalContext)
                    && !subExpressionData.contexts.contains(currentContext))))) {
            subExpressionData.contexts.push(currentContext);
        }
        if (this.parent != null) {
            subExpressionData.parent = this.parent;
        }
        else {
            subExpressionData = this;
        }
        subExpressionData.cacheEnabled = this.cacheEnabled;
        this.subExpressions.add(subExpressionData);
        return subExpressionData;
    }
    subExpressionData_Changed(context, propertyName, expression) {
        this.NotifyChanged(context, context, expression);
    }
    NotifyChanged(context, subContext, propertyName) {
        if ((this.Changed != null)) {
            this.Changed.invoke(context, subContext, propertyName, this.expression);
        }
        if ((this.parent != null)) {
            this.parent.NotifyChanged(context, subContext, propertyName);
        }
    }
    isBound(obj) {
        if (obj instanceof INotifyCollectionChanged_1.INotifyCollectionChanged) {
            this.boundCollections.forEach((item) => {
                if (item.checkBound(obj)) {
                    return true;
                }
            });
        }
        else if (obj instanceof INotifyPropertyChanged_1.INotifyPropertyChanged) {
            this.boundObjects.forEach((item) => {
                if (item.checkBound(obj)) {
                    return true;
                }
            });
            return false;
        }
        return false;
    }
    monitor(propertyName, bindableResult) {
        if (bindableResult instanceof INotifyPropertyChanged_1.INotifyPropertyChanged) {
            if (!this.isBound(bindableResult)) {
                let handler = new eventHandlers_1.PropertyChangedEventHandler();
                handler.add((sender, e) => {
                    if (propertyName != null && e.propertyName != null && (propertyName.equals(e.PropertyName) || propertyName.replace("[\"", "[").replace("\"]", "]").equals(e.PropertyName))) {
                        this.NotifyChanged(bindableResult, sender, e.propertyName);
                    }
                    else if (e.PropertyName != null && e.PropertyName.StartsWith(propertyName + "[")) {
                        this.NotifyChanged(bindableResult, sender, e.propertyName);
                    }
                });
                bindableResult.propertyChanged = handler;
                this.addHandler(bindableResult, handler);
            }
        }
        else if (bindableResult instanceof INotifyCollectionChanged_1.INotifyCollectionChanged) {
            let handler = new eventHandlers_1.NotifyCollectionChangedEventHandler();
            handler.add((sender, e) => {
                if (propertyName != null && e.propertyName != null && (propertyName.equals(e.PropertyName) || propertyName.replace("[\"", "[").replace("\"]", "]").equals(e.PropertyName))) {
                    this.NotifyChanged(bindableResult, sender, e.propertyName);
                }
                else if (e.PropertyName != null && e.PropertyName.StartsWith(propertyName + "[")) {
                    this.NotifyChanged(bindableResult, sender, e.propertyName);
                }
            });
            bindableResult.CollectionChanged = handler;
            this.addHandler(bindableResult, handler);
        }
    }
    addHandler(bindableResult, handler) {
        if (bindableResult instanceof INotifyPropertyChanged_1.INotifyPropertyChanged) {
            let objectHandler = new ObjectHandlerRecord();
            objectHandler.Handler = new eventHandlers_1.PropertyChangedEventHandler;
            objectHandler.Object = bindableResult;
            this.boundObjects.add(objectHandler);
        }
        else if (bindableResult instanceof INotifyCollectionChanged_1.INotifyCollectionChanged) {
            let boundCollections = new CollectionHandlerRecord();
            boundCollections.Handler = handler;
            boundCollections.Object = bindableResult;
            this.boundCollections.add(boundCollections);
        }
    }
    UnBindHandlers() {
        while ((this.boundObjects.count > 0)) {
            (new List_1.default(this.boundObjects)).forEach((item) => {
                this.boundObjects.clear();
            });
        }
        while ((this.boundCollections.count > 0)) {
            (new List_1.default(this.boundCollections)).forEach((item) => {
                item.unbind();
            });
            this.boundCollections.clear();
        }
    }
    equals(obj) {
        if ((obj instanceof expressionData)) {
            let ed = (obj);
            return (this.expression == ed.expression);
        }
        return false;
    }
    getHashCode() {
        let hash = 13;
        //  hash = (hash * 7) + (this.Binding != null ? this.Binding.GetHashCode() : 0);
        hash = ((hash * 7) + Utility_1.getHashCode(this.expression));
        return hash;
    }
    toLegacyIfNeeded() {
        if ((!this.isAlreadyLegacy
            && this.legacyMode)) {
            this.isAlreadyLegacy = true;
            this.expression = expressionCompiler_1.expressionCompiler.ProcessLegacy(this.expression);
        }
    }
    Dispose() {
        this.Destroy();
    }
    Destroy() {
        this.contexts.clear();
        // this.resultCache.clear();
        this.UnBindHandlers();
        // this.DestroySubParts();
        // ControlDestroyed = (ControlDestroyed - this.ExpressionData_ControlDestroyed);
        // this.globalContext = null;
        // this.pageContext = null;
    }
    // public DestroySubParts() {
    //     while ((this.subExpressions.count > 0)) {
    //         (new List<expressionData>(this.subExpressions)).forEach((item) => {
    //             let subExpression: expressionData;
    //             if (item.tryGetTarget(subExpression)) {
    //                 subExpression.Destroy();
    //             }
    //         });
    //         this.subExpressions.clear();
    //     }
    // }
    toString() {
        return this.expression;
    }
    // constructor() {
    //     this.Dispose();
    // }
    monitorIfRequired(item, currentContext) {
        let propertyName = "";
        if (this.monitorExpression) {
            if ((currentContext instanceof INotifyCollectionChanged_1.INotifyCollectionChanged)) {
                this.monitor("", currentContext);
            }
            if ((currentContext instanceof INotifyPropertyChanged_1.INotifyPropertyChanged && (item != null))) {
                if ((item instanceof propertyPart_1.propertyPart)) {
                    propertyName = (item).propertyName;
                }
                else if ((item instanceof indexerPart_1.indexerPart)) {
                    propertyName = item.toString();
                }
                if ((propertyName != null)) {
                    this.monitor(propertyName, (currentContext));
                }
            }
        }
    }
}
exports.default = expressionData;
class HandlerRecord {
}
class ObjectHandlerRecord extends HandlerRecord {
    unbind() {
        // let obj: INotifyPropertyChanged;
        // let handler: PropertyChangedEventHandler;
        // if (this.Object.TryGetTarget(/* out */obj)) {
        //     if (this.Handler.TryGetTarget(/* out */handler)) {
        //         obj.PropertyChanged = (obj.PropertyChanged - handler);
        //     }
        // }
    }
    checkBound(obj) {
        let tryObj;
        if (this.Object instanceof INotifyPropertyChanged_1.INotifyPropertyChanged) {
            return true;
        }
        return false;
    }
}
class CollectionHandlerRecord extends HandlerRecord {
    unbind() {
        // let obj: INotifyCollectionChanged;
        // let handler: NotifyCollectionChangedEventHandler;
        // if (this.Object.TryGetTarget(/* out */obj)) {
        //     if (this.Handler.TryGetTarget(/* out */handler)) {
        //         obj.CollectionChanged = (obj.CollectionChanged - handler);
        //     }
        // }
    }
    checkBound(obj) {
        let tryObj;
        if (this.Object instanceof INotifyCollectionChanged_1.INotifyCollectionChanged) {
            return true;
        }
        return false;
    }
}
//# sourceMappingURL=expressionData.js.map