"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toLocalTime = exports.toUtc = exports.addPeriod = exports.utcNow = exports.now = exports.isToday = exports.isDate = void 0;
const mfunction_1 = require("../mfunction");
const DateTime_1 = require("typescript-dotnet-commonjs/System/Time/DateTime");
const TimeSpan_1 = require("typescript-dotnet-commonjs/System/Time/TimeSpan");
const utils_1 = require("./../../compiler/utils");
const outputVariables_1 = require("../../utils/outputVariables");
class isDate extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 2;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        resolved.resolved = false;
        let value = undefined;
        let matchYear = true;
        if (parameters.count > 0) {
            value = (parameters.count > 0) ? parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ running) : DateTime_1.DateTime.now.toString();
            if ((parameters.count > 1)) {
                matchYear = parameters.get(1).resolve(expressionData, /* out */ resolved, /* running:*/ running);
            }
        }
        else {
            resolved.resolved = false;
        }
        let paramDate = undefined;
        if (resolved.resolved && obj != null && (value instanceof DateTime_1.DateTime || typeof value == "string")) {
            let paramIsDate = false;
            if (value instanceof DateTime_1.DateTime) {
                paramDate = (value);
                paramIsDate = true;
            }
            else {
                paramDate = new DateTime_1.DateTime(Date.parse(value === null || value === void 0 ? void 0 : value.toString()));
            }
            return (paramIsDate && utils_1.utils.matchDates(paramDate, (obj), matchYear));
        }
        resolved.resolved = false;
        return false;
    }
}
exports.isDate = isDate;
class isToday extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        resolved.resolved = true;
        let matchYear = true;
        if (parameters.count > 0) {
            matchYear = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ running);
        }
        return (obj instanceof DateTime_1.DateTime && utils_1.utils.matchDates(DateTime_1.DateTime.now, (obj), matchYear));
    }
}
exports.isToday = isToday;
class now extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        resolved.resolved = true;
        return Date.parse(DateTime_1.DateTime.now.toJsDate().toString());
    }
}
exports.now = now;
class utcNow extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        resolved.resolved = true;
        return Date.parse(DateTime_1.DateTime.now.toUniversalTime().toJsDate().toString());
    }
}
exports.utcNow = utcNow;
class addPeriod extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 2;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        let subResolved = new outputVariables_1.outputVariables();
        let val = parameters.count > 0 ? parameters.get(0).resolve(expressionData, /* out */ subResolved) : 0;
        let type = parameters.count > 1 ? parameters.get(1).resolve(expressionData, /* out */ subResolved) : "";
        let addition = TimeSpan_1.TimeSpan.fromTicks(0);
        if ((obj != null)) {
            switch (type.toLowerCase()) {
                case "day":
                case "days":
                    addition = TimeSpan_1.TimeSpan.fromDays(val);
                    break;
                case "week":
                case "weeks":
                    addition = TimeSpan_1.TimeSpan.fromDays((val * 7));
                    break;
                case "month":
                case "months":
                    if (obj instanceof DateTime_1.DateTime || obj instanceof DateTime_1.DateTime) {
                        obj = (obj).addMonths((val));
                    }
                    else {
                        addition = TimeSpan_1.TimeSpan.fromDays((30 * val));
                    }
                    break;
                case "yr":
                case "yrs":
                case "year":
                case "years":
                    if (obj instanceof DateTime_1.DateTime || obj instanceof DateTime_1.DateTime) {
                        obj = (obj).addYears((val));
                    }
                    else {
                        addition = TimeSpan_1.TimeSpan.fromDays((365 * val));
                    }
                    break;
                case "hr":
                case "hrs":
                case "hour":
                case "hours":
                    addition = TimeSpan_1.TimeSpan.fromHours(val);
                    break;
                case "min":
                case "mins":
                case "minute":
                case "minutes":
                    addition = TimeSpan_1.TimeSpan.fromMinutes(val);
                    break;
                case "sec":
                case "secs":
                case "second":
                case "seconds":
                    addition = TimeSpan_1.TimeSpan.fromSeconds(val);
                    break;
                case "ms":
                case "millis":
                case "millisec":
                case "millisecs":
                case "millisecond":
                case "milliseconds":
                    addition = TimeSpan_1.TimeSpan.fromMilliseconds(val);
                    break;
                case "tick":
                case "ticks":
                    break;
                default:
                    addition = TimeSpan_1.TimeSpan.fromTicks(val);
                    break;
            }
            if (obj instanceof DateTime_1.DateTime || obj instanceof DateTime_1.DateTime) {
                obj = (obj).add(addition);
                resolved.resolved = true;
            }
            else if (obj instanceof TimeSpan_1.TimeSpan || obj instanceof TimeSpan_1.TimeSpan) {
                obj = (obj).add(addition);
                resolved.resolved = true;
            }
            else {
                resolved.resolved = false;
            }
        }
        else {
            resolved.resolved = false;
        }
        return obj;
    }
}
exports.addPeriod = addPeriod;
class toUtc extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if (obj instanceof DateTime_1.DateTime || obj instanceof DateTime_1.DateTime) {
            obj = (obj).toUniversalTime();
            resolved.resolved = true;
        }
        else {
            resolved.resolved = false;
        }
        return obj;
    }
}
exports.toUtc = toUtc;
class toLocalTime extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if (obj instanceof Date || obj instanceof Date) {
            obj = (obj).toLocaleTimeString();
            resolved.resolved = true;
        }
        else if (obj instanceof DateTime_1.DateTime || obj instanceof DateTime_1.DateTime) {
            obj = (obj).toJsDate().toLocaleTimeString();
            resolved.resolved = true;
        }
        else {
            resolved.resolved = false;
        }
        return obj;
    }
}
exports.toLocalTime = toLocalTime;
//# sourceMappingURL=date.js.map