"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toObjectList = exports.toObject = exports.coalesce = exports.toNumber = exports.toDate = exports.toDictionary = exports.toList = void 0;
const mfunction_1 = require("./../mfunction");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const literalPart_1 = require("../../compiler/parts/literalPart");
const DateTime_1 = require("typescript-dotnet-commonjs/System/Time/DateTime");
const reflectionExtender_1 = require("../../utils/reflectionExtender");
const typeMapper_1 = require("../../types/typeMapper");
const namedSelection_1 = require("../namedSelection");
const outputVariables_1 = require("../../utils/outputVariables");
const general_1 = require("../../utils/general");
const selection_1 = require("../selection");
const dictionaryObject_1 = require("../../utils/dictionaryObject");
class toList extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return -1;
    }
    compileParameters(paramList, flags) {
        if (flags.legacyMode) {
            let parts = new List_1.List();
            for (let param of paramList.toArray()) {
                let lPart = new literalPart_1.literalPart();
                lPart.value = param.trim();
                parts.add(lPart);
            }
            return parts;
        }
        else {
            return super.compileParameters(paramList, flags);
        }
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        resolved.resolved = true;
        let result = new List_1.List();
        for (let ex of parameters.toArray()) {
            let str = ex.resolve(expressionData, resolved, running);
            if (str !== undefined)
                result.add(str);
        }
        resolved.resolved = true;
        return result;
    }
}
exports.toList = toList;
class toDictionary extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return -1;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        resolved.resolved = true;
        let result = new dictionaryObject_1.DictionaryObject();
        for (let ex of parameters.toArray()) {
            let object;
            let val = (ex.resolveGeneric(expressionData, resolved, reflectionExtender_1.getType(new dictionaryObject_1.DictionaryObject()), running));
            let key = val.getKeys()[0];
            let valueOfDic = val.getValue(key);
            result.add(key, valueOfDic);
        }
        resolved.resolved = true;
        return result;
    }
}
exports.toDictionary = toDictionary;
class toDate extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 1;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let value = null;
        resolved.resolved = true;
        if ((parameters.count > 0)) {
            value = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ undefined);
        }
        let paramDate;
        if ((resolved
            && (value != null))) {
            let paramIsDate = false;
            if ((value instanceof DateTime_1.DateTime)) {
                paramDate = (value);
                paramIsDate = true;
            }
            else {
                //paramIsDate =  DateTime.TryParse(value.toString(), /* out */paramDate);
                paramDate = new DateTime_1.DateTime(value.toString());
                paramIsDate = true;
            }
            if (paramIsDate) {
                //Object.values[]
                return Date.parse(Object.values(paramDate)[1]);
            }
        }
        return undefined;
    }
}
exports.toDate = toDate;
class toNumber extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 2;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let value = undefined;
        resolved.resolved = true;
        let type = "auto";
        if ((parameters.count > 0)) {
            value = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ undefined);
        }
        if ((parameters.count > 1)) {
            type = parameters.get(1).resolve(expressionData, /* out */ resolved, /* running:*/ undefined);
        }
        if ((resolved.resolved
            && (value != null))) {
            let str = value.toString();
            switch (type) {
                // case "sbyte":
                //     return typeMapper.toType<System.SByte>(value);
                //     break;
                case "short":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "int":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "long":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "byte":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "ushort":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "uint":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "ulong":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "float":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "double":
                    return typeMapper_1.typeMapper.toType(value);
                    break;
                case "auto":
                default:
                    let tempInt;
                    let d = value;
                    if (general_1.isNumberType(value) && general_1.isFloatType(value)) {
                        return Number.parseFloat(value.toString());
                    }
                    else if (general_1.isNumberType(value)) {
                        return Number.parseInt(value.toString());
                    }
                    else
                        return value;
                    // if (number.TryParse((value + ""), /* out */tempInt)) {
                    //     return tempInt;
                    // }
                    // else {
                    //     return typeMapper.toType<number>(value);
                    // }
                    break;
            }
        }
        else {
            switch (type) {
                case "short":
                    return (0);
                    break;
                case "int":
                    return (0);
                    break;
                case "long":
                    return (0);
                    break;
                case "byte":
                    return (0);
                    break;
                case "ushort":
                    return (0);
                    break;
                case "uint":
                    return (0);
                    break;
                case "ulong":
                    return (0);
                    break;
                case "float":
                    return (0);
                    break;
                case "double":
                    return (0);
                    break;
                case "auto":
                    break;
                default:
                    return 0;
                    break;
            }
        }
    }
}
exports.toNumber = toNumber;
class coalesce extends mfunction_1.mfunction {
    get minParameters() {
        return 2;
    }
    get maxParameters() {
        return 3;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        resolved.resolved = true;
        if ((parameters.count > 1)) {
            let subResolved = new outputVariables_1.outputVariables();
            let value = parameters.get(0).resolve(expressionData, /* out */ subResolved, /* running:*/ undefined);
            let emptyAsNull = parameters.count > 2 ? parameters.get(2).resolveGeneric(expressionData, subResolved, reflectionExtender_1.getType(Object), undefined) : false;
            if (subResolved.resolved && value != null && (!emptyAsNull || !(typeof value == 'string') || !String.isNullOrEmpty(value))) {
                return value;
            }
            else {
                return parameters.get(1).resolve(expressionData, subResolved, /* running:*/ undefined);
            }
        }
        return undefined;
    }
}
exports.coalesce = coalesce;
class toObject extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return -1;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        resolved.resolved = true;
        let source = undefined;
        if ((parameters.count == 1)) {
            let subResolved = new outputVariables_1.outputVariables();
            source = parameters.get(0).resolve(expressionData, /* out */ subResolved, /* running:*/ undefined);
        }
        else {
            source = expressionData.contexts.peek();
        }
        if ((source != null)) {
            if (source instanceof namedSelection_1.namedSelection || source instanceof dictionaryObject_1.DictionaryObject || source instanceof selection_1.selection) {
                let eo = {};
                let eoColl = new List_1.List();
                let dict = new dictionaryObject_1.DictionaryObject();
                if (source instanceof namedSelection_1.namedSelection) {
                    for (let strKey of source.getKeys()) {
                        dict.add(strKey.toString(), source.getValue(strKey));
                    }
                }
                else if (source instanceof selection_1.selection) {
                    let num = 0;
                    for (let strKey of source.toArray()) {
                        dict.add(num.toString(), strKey);
                        num++;
                    }
                }
                else {
                    dict = source;
                }
                for (let kvp of dict.getKeys()) {
                    //eoColl.add({ key: kvp, value: dict.getValue(kvp) as Object });
                    eo[kvp] = dict.getValue(kvp);
                }
                return (eo);
            }
            else {
                return source;
            }
        }
        return undefined;
    }
}
exports.toObject = toObject;
class toObjectList extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return -1;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        resolved.resolved = true;
        let source = undefined;
        let func = new toObject();
        if ((parameters.count == 1)) {
            let subResolved = new outputVariables_1.outputVariables();
            source = parameters.get(0).resolve(expressionData, /* out */ subResolved, /* running:*/ undefined);
        }
        else {
            source = expressionData.contexts.peek();
        }
        if ((source != null)) {
            if (source.getEnumerator() !== undefined) {
                let newList = new List_1.List();
                for (let item of (source).toArray()) {
                    expressionData.contexts.push(item);
                    let subResolved = new outputVariables_1.outputVariables();
                    newList.add(func.invoke(expressionData, new List_1.List(), /* out */ subResolved, running));
                    expressionData.contexts.pop();
                }
                return newList;
            }
            return source;
        }
        return undefined;
    }
}
exports.toObjectList = toObjectList;
//# sourceMappingURL=literals.js.map