"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.regexpReplace = exports.matches = exports.contains = exports.concat = exports.concatNonEmpty = exports.stringformat = exports.toString = exports.toLower = exports.toUpper = exports.split = exports.replace = exports.substring = void 0;
const mfunction_1 = require("../mfunction");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const RegularExpressions_1 = require("typescript-dotnet-commonjs/System/Text/RegularExpressions");
const outputVariables_1 = require("../../utils/outputVariables");
const dotnet = require("typescript-dotnet-commonjs/System/Text/Utility");
const DateTime_1 = require("typescript-dotnet-commonjs/System/Time/DateTime");
const date_fns_1 = require("date-fns");
class substring extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 2;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if (obj != null && typeof obj == "string") {
            if ((parameters.count == 1)) {
                let startIndex = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ running);
                if (resolved) {
                    return (obj).substring(startIndex);
                }
            }
            else if ((parameters.count >= 2)) {
                let startIndex = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ running);
                let length = parameters.get(1).resolve(expressionData, /* out */ resolved, /* running:*/ running);
                if (resolved) {
                    let str = (obj);
                    if (((startIndex >= 0)
                        && (str.length
                            >= (startIndex + length)))) {
                        return str.substring(startIndex, length);
                    }
                    else {
                        return str;
                    }
                }
            }
        }
        resolved.resolved = false;
        return obj;
    }
}
exports.substring = substring;
class replace extends mfunction_1.mfunction {
    get minParameters() {
        return 2;
    }
    get maxParameters() {
        return 2;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if (obj != null && typeof obj == "string" && parameters.count >= 2) {
            let str1 = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ running);
            let str2 = parameters.get(1).resolve(expressionData, /* out */ resolved, /* running:*/ running);
            if (resolved) {
                return (obj).replace(str1, str2);
            }
        }
        resolved.resolved = false;
        return obj;
    }
}
exports.replace = replace;
class split extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 1;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if (obj != null && typeof obj == "string" && parameters.count >= 1) {
            let str1 = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ running);
            if (resolved) {
                return (obj).split(str1);
            }
        }
        resolved.resolved = false;
        return obj;
    }
}
exports.split = split;
class toUpper extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if (obj != null && typeof obj == 'string') {
            resolved.resolved = true;
            return (obj).toUpperCase();
        }
        resolved.resolved = false;
        return obj;
    }
}
exports.toUpper = toUpper;
class toLower extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if ((obj != null)
            && typeof obj == 'string') {
            resolved.resolved = true;
            return (obj).toLowerCase();
        }
        resolved.resolved = false;
        return obj;
    }
}
exports.toLower = toLower;
class toString extends mfunction_1.mfunction {
    get minParameters() {
        return 0;
    }
    get maxParameters() {
        return 0;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        resolved.resolved = true;
        return obj.toString();
        // TODO: Warning!!!, inline IF is not supported ?
        (obj != null);
        "";
    }
}
exports.toString = toString;
class stringformat extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 1;
    }
    dateFormat(dt, sformat) {
        sformat = sformat.replace('0:', '').
            replace('{', '').
            replace('}', '').replace('tt', 'aaa').replace('dddd', 'EEEE').replace('ddd', 'eee');
        return date_fns_1.format(dt, sformat);
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if (((obj != null)
            && (parameters.count == 1))) {
            let sformat = parameters.get(0).resolve(expressionData, /* out */ resolved, /* running:*/ running);
            resolved.resolved = true;
            if (obj instanceof DateTime_1.default || obj instanceof Date) {
                const dt = obj instanceof DateTime_1.default ? obj.toJsDate() : obj; // let d = Date();
                return this.dateFormat(dt, sformat);
            }
            else if (typeof obj === 'number' || typeof obj === 'string') {
                const dt = new Date(obj);
                if (!isNaN(dt.getTime())) {
                    return this.dateFormat(dt, sformat);
                }
            }
            else {
                return dotnet.format(sformat, obj);
            }
        }
        resolved.resolved = false;
        return obj;
    }
}
exports.stringformat = stringformat;
class concatNonEmpty extends mfunction_1.mfunction {
    get minParameters() {
        return 2;
    }
    get maxParameters() {
        return -1;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if ((parameters.count > 1)) {
            let items = new List_1.List(); //(parameters.count - 1);
            let subResolved = new outputVariables_1.outputVariables();
            let delimiter = parameters.get(0).resolve(expressionData, /* out */ subResolved, /* running:*/ running);
            if (delimiter = "" && delimiter != null) // String.isNullOrEmpty(delimiter)) 
             {
                delimiter = ",";
            }
            for (let i = 1; (i < parameters.count); i++) {
                let subParamResolved = new outputVariables_1.outputVariables();
                let val = parameters.get(i).resolve(expressionData, /* out */ subParamResolved, /* running:*/ running);
                if ((subParamResolved
                    && ((val != null)
                        && (!(typeof val == 'string')
                            || !String.isNullOrEmpty((val)))))) {
                    items.add(val);
                }
            }
            if (subResolved.resolved) {
                resolved.resolved = true;
                if (items.count == 1 && items.get(0) instanceof List_1.List && !(typeof items.get(0) == 'string')) {
                    return items.get(0).toArray().join(delimiter);
                }
                else {
                    return items.toArray().join(delimiter);
                }
            }
            resolved.resolved = false;
            return obj;
        }
    }
}
exports.concatNonEmpty = concatNonEmpty;
class concat extends mfunction_1.mfunction {
    get minParameters() {
        return 2;
    }
    get maxParameters() {
        return -1;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let obj = expressionData.contexts.peek();
        if ((parameters.count > 1)) {
            let items = new Array((parameters.count - 1));
            let subResolved = new outputVariables_1.outputVariables();
            let delimiter = parameters.get(0).resolve(expressionData, /* out */ subResolved, /* running:*/ running);
            if (delimiter != "" && delimiter != null) //string.IsNullOrEmpty(delimiter)) 
             {
                delimiter = ",";
            }
            for (let i = 1; (i < parameters.count); i++) {
                let subParamResolved = new outputVariables_1.outputVariables();
                items[i - 1] = parameters.get(i).resolve(expressionData, /* out */ subParamResolved, /* running:*/ running);
                subResolved = (subResolved || subParamResolved);
            }
            if (subResolved) {
                resolved.resolved = true;
                if (items.length == 1 && items[0] instanceof List_1.List && !(typeof items[0] == 'string')) {
                    return items[0].toArray().join(delimiter); //string.Join(delimiter, items[0] as IEnumerable<object>);
                }
                else {
                    return items.join(delimiter);
                }
            }
        }
        resolved.resolved = false;
        return obj;
    }
}
exports.concat = concat;
class contains extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 2;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let context = expressionData.contexts.peek();
        if ((parameters.count > 0)) {
            let subResolved = new outputVariables_1.outputVariables();
            let obj = context;
            let param = parameters.get(0).resolve(expressionData, /* out */ subResolved, /* running:*/ running);
            if ((parameters.count > 1)) {
                obj = param;
                param = parameters.get(1).resolve(expressionData, /* out */ subResolved, /* running:*/ running);
            }
            if (subResolved) {
                if (typeof obj == 'string' && typeof param == 'string') { //|| param instanceof char
                    resolved.resolved = true;
                    let strObj = (obj);
                    if (!String.isNullOrEmpty(param)) {
                        return strObj.indexOf(param) >= 0;
                    }
                    // else if (typeof param === 'char') {
                    //     return strObj.indexOf(param.toString())>0;
                    // }
                }
                else if (obj instanceof List_1.List) {
                    resolved.resolved = true;
                    for (let item of (obj).toArray()) {
                        if (((item != null)
                            && item.equals(param))) {
                            return true;
                        }
                        else if ((item == param)) {
                            return true;
                        }
                    }
                }
            }
        }
        resolved.resolved = false;
        return context;
    }
}
exports.contains = contains;
class matches extends mfunction_1.mfunction {
    get minParameters() {
        return 1;
    }
    get maxParameters() {
        return 2;
    }
    invoke(expressionDataObj, parameters, /* out */ resolved, running) {
        let context = expressionDataObj.contexts.peek();
        if ((parameters.count > 0)) {
            let subResolved = new outputVariables_1.outputVariables();
            let obj = (context);
            let regex = parameters.get(0).resolve(expressionDataObj, /* out */ subResolved, /* running:*/ running);
            let ro = RegularExpressions_1.RegexOptions.GLOBAL; // RegexOptions.None;
            if ((parameters.count > 1)) {
                let param = parameters.get(1).resolve(expressionDataObj, /* out */ subResolved, /* running:*/ running);
                if (param.indexOf('m') > -1) {
                    ro = RegularExpressions_1.RegexOptions.MULTI_LINE; //(ro | RegexOptions.MULTI_LINE);
                }
                if (param.indexOf('i') > -1) {
                    ro = RegularExpressions_1.RegexOptions.IGNORE_CASE; //(ro | RegexOptions.IGNORE_CASE);
                }
                if (param.indexOf('w') > -1) {
                    ro = RegularExpressions_1.RegexOptions.IGNORE_PATTERN_WHITESPACE; //(ro | RegexOptions.IGNORE_PATTERN_WHITESPACE);
                }
                // if (param.Contains('s')) {
                //     ro = (ro | RegexOptions.SINGLE_LINE);
                // }
                /*if (param.indexOf('r')>-1) {
                    ro = //(ro | RegexOptions.RightToLeft);
                }
                
                if (param.indexOf('e')>-1) {
                    ro = (ro | RegexOptions.ExplicitCapture);
                }*/
            }
            if ((subResolved.resolved && ((obj != null) && (regex != null)))) {
                resolved.resolved = true;
                if (ro != null) // RegexOptions.)
                 {
                    var regexLiteral = [ro];
                    var result = false;
                    let reg = new RegularExpressions_1.Regex(regex, RegularExpressions_1.RegexOptions.MULTI_LINE);
                    let d = reg.matches(obj);
                    result = (RegularExpressions_1.Regex.isMatch(obj, regex, regexLiteral));
                    return result;
                }
                else {
                    return (RegularExpressions_1.Regex.isMatch(obj, regex));
                }
            }
        }
        resolved.resolved = false;
        return false;
    }
}
exports.matches = matches;
class regexpReplace extends mfunction_1.mfunction {
    get minParameters() {
        return 2;
    }
    get maxParameters() {
        return 3;
    }
    invoke(expressionData, parameters, /* out */ resolved, running) {
        let context = expressionData.contexts.peek();
        if ((parameters.count > 1)) {
            let subResolved = new outputVariables_1.outputVariables();
            let obj = (context);
            let regex = parameters.get(0).resolve(expressionData, /* out */ subResolved, /* running:*/ running);
            let replace = parameters.get(1).resolve(expressionData, /* out */ subResolved, /* running:*/ running);
            let ro = []; //RegexOptions.None;
            let flags;
            if ((parameters.count > 2)) {
                let param = parameters.get(2).resolve(expressionData, /* out */ subResolved, /* running:*/ running);
                flags = param;
                if (param.indexOf('m') > 0) {
                    ro.push(RegularExpressions_1.RegexOptions.MULTI_LINE);
                }
                if (param.indexOf('i') > 0) {
                    ro.push(RegularExpressions_1.RegexOptions.IGNORE_CASE);
                }
                if (param.indexOf('w') > 0) {
                    ro.push(RegularExpressions_1.RegexOptions.IGNORE_PATTERN_WHITESPACE);
                }
                // if (param.indexOf('s')>0) {
                //     ro = (ro | RegexOptions.Singleline);
                // }
                // if (param.indexOf('r')>0) {
                //     ro = (ro | RegexOptions.RightToLeft);
                // }
                // if (param.indexOf('e')>0) {
                //     ro = (ro | RegexOptions.ExplicitCapture);
                // }
            }
            if ((subResolved.resolved
                && ((obj != null)
                    && (regex != null)))) {
                resolved.resolved = true;
                if ((ro != null)) {
                    var regexLiteral = ro;
                    let d = new RegExp(regex, flags);
                    let f = d.exec(replace);
                    d.test(replace);
                    return obj.replace(d, replace);
                }
                else {
                    return RegularExpressions_1.Regex.replace(obj, regex, replace);
                }
            }
        }
        resolved.resolved = false;
        return context;
    }
}
exports.regexpReplace = regexpReplace;
//# sourceMappingURL=string.js.map