"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.operatorsList = void 0;
const arithmeticOperator_1 = require("./arithmeticOperator");
const boolOperator_1 = require("./boolOperator");
const compareOperator_1 = require("./compareOperator");
const generalOperator_1 = require("./generalOperator");
const List_1 = require("typescript-dotnet-commonjs/System/Collections/List");
const Utility_1 = require("typescript-dotnet-commonjs/System/Text/Utility");
const utils_1 = require("../compiler/utils");
const typeMapper_1 = require("../types/typeMapper");
const DateTime_1 = require("typescript-dotnet-commonjs/System/Time/DateTime");
const dictionaryObject_1 = require("../utils/dictionaryObject");
class operatorsList {
    static get binaryOperators() {
        if (this._binaryOperators == undefined) {
            this._binaryOperators = new dictionaryObject_1.DictionaryObject();
            this.initOperator();
        }
        return this._binaryOperators;
    }
    static initOperator() {
        let arithmeticOp = new arithmeticOperator_1.arithmeticOperator();
        arithmeticOp.precedence = 108;
        arithmeticOp.name = "+";
        arithmeticOp.opInvoke = (v) => {
            if ((v[0] instanceof List_1.default && typeof v[1] == 'string') || (typeof v[0] == 'string' && v[1] instanceof List_1.default)) {
                let result = new List_1.default();
                let sourceList = (v[0] instanceof List_1.default) ? v[0] : v[1];
                let sourceString = (typeof v[0] == 'string') ? v[0] : v[1];
                for (let obj of sourceList.toArray()) {
                    result.add(Utility_1.format("{0}{1}", obj, sourceString));
                }
                return result;
            }
            else if (v[0] instanceof List_1.default || v[1] instanceof List_1.default) {
                let result = new List_1.default();
                let index = 0;
                for (var obj of v[0].toArray()) {
                    result.add(Utility_1.format("{0}{1}", obj, v[1].toArray()[index]));
                    index++;
                }
                return result;
            }
            else if (v[0] instanceof DateTime_1.default && v[1] instanceof DateTime_1.default) {
                return Utility_1.format("{0}{1}", v[0].toJsDate().toString(), v[1].toJsDate().toString());
            }
            else if (v[0] instanceof DateTime_1.default || v[1] instanceof DateTime_1.default) {
                if (v[0] instanceof DateTime_1.default)
                    return Utility_1.format("{0}{1}", v[0] === undefined ? '' : v[0].toJsDate().toString(), v[1] === undefined ? '' : v[1]);
                if (v[1] instanceof DateTime_1.default)
                    return Utility_1.format("{0}{1}", v[0] === undefined ? '' : v[0], v[1] === undefined ? '' : v[1].toJsDate().toString());
            }
            else if (typeof v[0] == 'string' || typeof v[1] == 'string') {
                return Utility_1.format("{0}{1}", v[0] === undefined ? '' : v[0], v[1] === undefined ? '' : v[1]);
            }
            else if (utils_1.utils.supportsOperators(v[0]) && utils_1.utils.supportsOperators(v[1])) {
                return utils_1.utils.toBiggerTypeOperation(v[0], v[1], '+');
            }
            return undefined;
        };
        let arithmeticOpMinus = new arithmeticOperator_1.arithmeticOperator();
        arithmeticOpMinus.precedence = 108;
        arithmeticOpMinus.name = "-";
        arithmeticOpMinus.opInvoke = (v) => {
            if (utils_1.utils.supportsOperators(v[0]) && utils_1.utils.supportsOperators(v[1])) {
                return utils_1.utils.toBiggerTypeOperation(v[0], v[1], '-');
            }
            return undefined;
        };
        let arithmeticOpMultiply = new arithmeticOperator_1.arithmeticOperator();
        arithmeticOpMultiply.precedence = 109;
        arithmeticOpMultiply.name = "*";
        arithmeticOpMultiply.opInvoke = (v) => {
            if (utils_1.utils.supportsOperators(v[0]) && utils_1.utils.supportsOperators(v[1])) {
                return utils_1.utils.toBiggerTypeOperation(v[0], v[1], '*');
            }
            return null;
        };
        let arithmeticOpDivide = new arithmeticOperator_1.arithmeticOperator();
        arithmeticOpDivide.precedence = 109;
        arithmeticOpDivide.name = "/";
        arithmeticOpDivide.opInvoke = (v) => {
            if (utils_1.utils.supportsOperators(v[0]) && utils_1.utils.supportsOperators(v[1])) {
                return utils_1.utils.toBiggerTypeOperation(v[0], v[1], '/');
            }
            return null;
        };
        let arithmeticOpPercentage = new arithmeticOperator_1.arithmeticOperator();
        arithmeticOpPercentage.precedence = 109;
        arithmeticOpPercentage.name = "%";
        arithmeticOpPercentage.opInvoke = (v) => {
            if (utils_1.utils.supportsOperators(v[0]) && utils_1.utils.supportsOperators(v[1])) {
                return utils_1.utils.toBiggerTypeOperation(v[0], v[1], '%');
            }
            return null;
        };
        let arithmeticOp104 = new arithmeticOperator_1.arithmeticOperator();
        arithmeticOp104.precedence = 104;
        arithmeticOp104.name = "^";
        arithmeticOp104.opInvoke = (v) => {
            if (utils_1.utils.supportsOperators(v[0]) && utils_1.utils.supportsOperators(v[1])) {
                return utils_1.utils.toBiggerTypeOperation(v[0], v[1], '^');
            }
            else if (typeof v[0] == 'boolean' && typeof v[1] == 'boolean') {
                //return (typeMapper.toType<boolean>(v[0]) as boolean) ^ (typeMapper.toType<boolean>(v[1]));
            }
            return null;
        };
        arithmeticOp104.canCutOff = undefined;
        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp,
            arithmeticOpMinus, arithmeticOpMultiply, arithmeticOpDivide, arithmeticOpPercentage, arithmeticOp104]);
        let arithmeticOp105 = new boolOperator_1.boolOperator();
        arithmeticOp105.precedence = 105;
        arithmeticOp105.name = "&";
        arithmeticOp105.opInvoke = (v) => {
            let res = (v[0] && v[1]);
            return res;
        };
        arithmeticOp105.canCutOff = undefined;
        // arithmeticOp105.canCutOff =  ( currentTruth:boolean) =>
        // {
        //     return !currentTruth;
        // };
        let arithmeticOp103 = new boolOperator_1.boolOperator();
        arithmeticOp103.precedence = 103;
        arithmeticOp103.name = "|";
        arithmeticOp103.opInvoke = (v) => {
            let res = (v[0] || v[1]);
            return res;
        };
        arithmeticOp105.canCutOff = undefined;
        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp105, arithmeticOp103]);
        let arithmeticOp106 = new compareOperator_1.compareOperator();
        arithmeticOp106.precedence = 106;
        arithmeticOp106.name = "=";
        arithmeticOp106.opInvoke = (v) => {
            return utils_1.utils.compare(v[0], v[1], '=');
        };
        let arithmeticOp1061 = new compareOperator_1.compareOperator();
        arithmeticOp1061.precedence = 106;
        arithmeticOp1061.name = "!";
        arithmeticOp1061.opInvoke = (v) => {
            return utils_1.utils.compare(v[0], v[1], '!');
        };
        let arithmeticOp1062 = new compareOperator_1.compareOperator();
        arithmeticOp1062.precedence = 106;
        arithmeticOp1062.name = "~";
        arithmeticOp1062.opInvoke = (v) => {
            var _a;
            if (typeof v[0] == 'string' && typeof v[1] == 'string') {
                let strv1 = v[0];
                let strv2 = v[1];
                return strv2.indexOf(strv1) > -1;
            }
            if (typeof v[0] == 'number' && typeof v[1] == 'number') {
                let strv1 = v[0].toString();
                let strv2 = v[1].toString();
                return strv2.indexOf(strv1) > -1;
            }
            else if (v[1] instanceof List_1.default) {
                let ie = v[1].getEnumerator();
                while (ie.moveNext()) {
                    if ((_a = ie.current) === null || _a === void 0 ? void 0 : _a.equals(v[0]))
                        return true;
                }
            }
            return false;
        };
        let arithmeticOp1063 = new compareOperator_1.compareOperator();
        arithmeticOp1063.precedence = 106;
        arithmeticOp1063.name = "#";
        arithmeticOp1063.opInvoke = (v) => {
            return v[0] == v[1] || (v[0] != null && v[1] != null && v[1].toString().indexOf(v[0].toString()) >= 0);
        };
        let arithmeticOp107 = new compareOperator_1.compareOperator();
        arithmeticOp107.precedence = 107;
        arithmeticOp107.name = "<";
        arithmeticOp107.opInvoke = (v) => {
            return utils_1.utils.compare(v[0], v[1], '<');
        };
        let arithmeticOp1071 = new compareOperator_1.compareOperator();
        arithmeticOp1071.precedence = 107;
        arithmeticOp1071.name = ">";
        arithmeticOp1071.opInvoke = (v) => {
            return utils_1.utils.compare(v[0], v[1], '>');
        };
        operatorsList.addOperators(operatorsList._binaryOperators, [arithmeticOp106, arithmeticOp1061, arithmeticOp1062, arithmeticOp1063, arithmeticOp107, arithmeticOp1071]);
        let arithmeticOp100 = new generalOperator_1.generalOperator();
        arithmeticOp100.precedence = 100;
        arithmeticOp100.name = ":";
        arithmeticOp100.opInvoke = (v) => {
            let dict = new dictionaryObject_1.DictionaryObject();
            dict.setValue(typeMapper_1.typeMapper.toType1(v[0]), v[1]);
            return dict;
        };
        operatorsList.addOperator(operatorsList._binaryOperators, arithmeticOp100);
        let arithmeticOp1001 = new generalOperator_1.generalOperator();
        arithmeticOp1001.precedence = 100;
        arithmeticOp1001.name = "@";
        arithmeticOp1001.opInvoke = (v) => {
            let result = new dictionaryObject_1.DictionaryObject();
            result.setValue(typeMapper_1.typeMapper.toType1(v[1]), v[0]);
            return result;
        };
        operatorsList.addOperator(operatorsList._binaryOperators, arithmeticOp1001);
    }
    static addOperator(target, op) {
        target.setValue(op.name, op);
    }
    static addOperators(target, ops) {
        for (let item of ops) {
            operatorsList.addOperator(target, item);
        }
    }
}
exports.operatorsList = operatorsList;
//# sourceMappingURL=operatorsList.js.map