﻿import { expressionPart } from "./expressionPart";
import { compoundPart } from './compoundPart';
import expressionData from "../../expressionData";
import { groupDataContext } from '../../functions/groupDataContext';
import { Stack } from "../../utils/stack";
import List from "typescript-dotnet-commonjs/System/Collections/List";
import IEnumerable from "typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerable";
import { outputVariables } from "../../utils/outputVariables";

export class partList extends compoundPart {

    private lastResolvedLevel: number = 0;

    protected parts: List<expressionPart> = new List<expressionPart>();

    public get Parts(): IEnumerable<expressionPart> {
        return this.parts;
    }

    public get simpleName(): string {
        return this.parts != null && this.parts.count > 0 ? this.parts.linq.last().simpleName : "Item";
    }

    public addSubPart(part: expressionPart): boolean {
        this.parts.add(part);
        return true;
    }

    resolve(expressionData: expressionData, /* out */resolved: outputVariables, running: groupDataContext, evaluateAllContexts: boolean = true, setValue: Object | undefined = undefined): Object | undefined {
        let contexts: Stack<Object>;
        if (evaluateAllContexts) {
            contexts = new Stack<Object>();
            let temp = expressionData.contexts.clone() as Stack<Object>;
            while (!temp.isEmpty()) {
                contexts.push(temp.pop());
            }
        }
        else {
            contexts = new Stack<Object>();
            contexts.push((expressionData.contexts as Stack<Object>).pop());
        }

        let parentData: expressionData = expressionData.parent ?? expressionData;;

        let tempData: expressionData = expressionData.toSubEx();

        if ((evaluateAllContexts
            && ((this.lastResolvedLevel > 0)
                && (this.lastResolvedLevel < contexts.count())))) {

        }

        while ((contexts.count() > 0)) {
            let currentContext: Object = contexts.pop();
            let subResolved = new outputVariables();
            let i: number = 0;
            for (; (i < this.parts.count); i++) {
                let item: expressionPart = this.parts.get(i);
                let popAtEnd: boolean = false;

                if (tempData.contexts.peek() != currentContext) {
                    tempData.contexts.push(currentContext);
                    popAtEnd = true;
                }

                parentData.monitorIfRequired(item, currentContext);
                if (i == (this.parts.count - 1)) {
                    currentContext = item.resolve(tempData, /* out */subResolved, running, false, setValue) as Object;
                    //  For observing observable collections
                    parentData.monitorIfRequired(undefined, currentContext);
                }
                else {
                    currentContext = item.resolve(tempData, /* out */subResolved, running, false, undefined) as Object;
                    if ((currentContext == null)) {
                        subResolved.resolved = false;
                    }

                }

                if (popAtEnd && !(<Stack<Object>>tempData.contexts).isEmpty()) {
                    (<Stack<Object>>tempData.contexts).pop();
                }

                if (!subResolved.resolved) {
                    break;
                }

            }
            if (!subResolved.resolved && i > 0) {
                resolved.resolved = false;

                return undefined;
            }

            if (subResolved.resolved) {
                resolved.resolved = true;
                this.lastResolvedLevel = (contexts.count() + 1);
                // parentData.resultCache.setValue(cacheKey, currentContext);
                return currentContext;
            }

        }

        resolved.resolved = false;
        return undefined;
    }

    public toString(): string {
        return this.parts.toArray().join(".");
    }
}
