﻿import { expressionPart } from './compiler/parts/expressionPart';
import expressionData from './expressionData';
import { expressionCompiler, compileFlags, ICompileFlags } from './compiler/expressionCompiler';
import { makeenBindingContext } from './bindingContext';
import List from 'typescript-dotnet-commonjs/System/Collections/List';
import IEnumerable from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerable';
import { format } from "typescript-dotnet-commonjs/System/Text/Utility";
import { propertyInfo, getType, getGenericType } from './utils/reflectionExtender';
import { typeMapper } from './types/typeMapper';
import { Type } from 'typescript';
import { outputVariables } from './utils/outputVariables';
import { ItemSourceContexts } from './utils/ItemSourceContexts';

export class expressionEngine {

    public static instance: expressionEngine;

    private static DEFAULT_LEGACY_MODE: boolean = true;

    public static getInstance(): expressionEngine {
        if ((this.instance == null)) {
            this.instance = new expressionEngine();
        }

        return this.instance;
    }

    public constructor() {

    }

    public static evalGeneric<T>(context: Object | undefined, expression: string, secondContext?: Object, globalContext?: Object, legacyMode: boolean = false, data?: expressionData, setValue: Object | undefined = undefined): any {
        let tType: Type | undefined;
        if (!expression) {
            return undefined;
        }

        if (tType == undefined)
            tType = getType(context);
        if (data != undefined) {
            return typeMapper.toType<T>(expressionEngine.eval(context, "", legacyMode, data, tType, setValue)) as T;
        }
        let result = expressionEngine.evalPrivate(context, expression, secondContext, globalContext, tType, legacyMode);

        return (typeMapper.toType1<T>(result, tType) as T);
    }

    public static eval(context: Object | undefined, expression: string, legacyMode: boolean = true, data?: expressionData, targetType?: Type, setValue: Object | undefined = undefined): Object {
        if (data == undefined) {
            return expressionEngine.evalPrivate(context, expression, undefined, undefined, getType(Object), legacyMode, null);
        }
        else {
            if (context != null && !data.contexts.contains(context))
                data.contexts.push(context);
            return expressionEngine.valueFor(context, data, targetType as Type, setValue) as Object;
        }
    }

    private static evalPrivate(context: Object | undefined, expression: string, secondContext?: Object, globalContext?: Object, targetType?: Type, legacyMode: boolean = true, setValue: Object | null = null): Object {
        let data: expressionData = new expressionData(expression, secondContext, globalContext, legacyMode);
        if ((context != null)) {
            data.contexts.push(context);
        }
        if (secondContext != null) {
            data.contexts.push(secondContext);
        }
        if (globalContext != null) {
            data.contexts.push(globalContext);
        }
        let result: Object | undefined = undefined;
        result = expressionEngine.eval(context, "", legacyMode, data, undefined, targetType);
        return result;
    }

    public static valueFor(subject: Object | undefined, data: expressionData, targetType: Type, setValue: Object | undefined): Object | null {
        try {
            let expression: expressionPart | undefined;
            let compileflag = new compileFlags();
            compileflag.legacyMode = data.legacyMode;
            expression = expressionCompiler.compile(data.expression, undefined, undefined, compileflag);

            let resolved: outputVariables = new outputVariables();
            resolved.resolved = true;
            if (data.contexts.peek() instanceof ItemSourceContexts) {
                let secondExpressionPart: expressionPart | undefined = undefined;
                const itemSourceContext = data.contexts.pop() as ItemSourceContexts;
                if (itemSourceContext.secondExpression) {
                    secondExpressionPart = expressionCompiler.compile(itemSourceContext.secondExpression, undefined, undefined, compileflag);
                }
                let results = [];
                let contexts = itemSourceContext.contexts instanceof List ? itemSourceContext.contexts.toArray() : itemSourceContext.contexts;
                for (const item of contexts) {
                    data.contexts.push(item);
                    let result: Object = expression?.resolveClass(data, resolved, targetType) as Object;
                    let result2: Object = secondExpressionPart?.resolveClass(data, resolved, targetType) as Object;

                    results.push({
                        result: result,
                        result2: result2,
                        context: item
                    });
                    data.contexts.pop();
                }
                // if (results.length !== 0)
                return results;
            }
            let result: Object = expression?.resolveClass(data, resolved, targetType) as Object;
            return result;
        }
        catch (e) {
            throw e;
        }

    }
}

export class expAnalyzer {

    count: number = 0;

    firstExecutionTime: number = 0;

    private _TotalExecutionTime: number = 0;

    public get TotalExecutionTime(): number {
        return this._TotalExecutionTime;
    }
    public set TotalExecutionTime(value: number) {
        this._TotalExecutionTime = value;
        this.averageExecutionTime = (this._TotalExecutionTime / this.count);
    }

    private averageExecutionTime: number = 0;

    public toString(): string {
        return format("Count:{0} First ExecutionTime:{1} Total ExecutionTime:{2} Average ExecutionTime:{3}", this.count, this.firstExecutionTime, this.TotalExecutionTime, this.averageExecutionTime);
    }
}