import {invalidParametersException} from './../error/invalidParametersException';
import expressionData from '../expressionData'
import {expressionPart} from '../compiler/parts/expressionPart'
import {expressionCompiler, ICompileFlags, compileFlags} from '../compiler/expressionCompiler'
import {List} from 'typescript-dotnet-commonjs/System/Collections/List'
import { outputVariables } from '../utils/outputVariables';

 export abstract class aggregateFunction {
        
        public get FunctionName(): string {
            return typeof this;
        }
        
        public /* virtual */ get minParameters(): number {
            return 1;
        }
        
        public /* virtual */ get maxParameters(): number {
            return 1;
        }
        
        public compileParameters(paramList: List<string>, flags: compileFlags): List<expressionPart> {
            if ((((this.minParameters > 0) 
                        && (paramList.count < this.minParameters)) 
                        || ((this.maxParameters >= 0) 
                        && (paramList.count > this.maxParameters)))) {
                throw new invalidParametersException(this.FunctionName, this.minParameters, this.maxParameters);
            }
            
            let parameters: List<expressionPart> = new List<expressionPart>();
            for (let item of paramList.toArray()) {
                parameters.add(expressionCompiler.compile(item,undefined,undefined, flags)as expressionPart);
            }
            
            return parameters;
        }
        
        public /* internal */ invoke(expressionData: expressionData, current: Object, running: Object, parameters: List<expressionPart>, /* out */resolved: outputVariables): Object {
            return this.invokeIteration(expressionData, current, running, parameters, /* out */resolved);
        }
        
        public /* internal */ abstract invokeIteration(expressionData: expressionData, current: Object, running: Object, parameters: List<expressionPart>, /* out */resolved: outputVariables): Object;
    }