﻿import expressionData from '../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { groupDataContext } from './../../functions/groupDataContext'
import { mfunction } from './../mfunction';
import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import { typeMapper } from './../../types/typeMapper'
import { getType } from '../../utils/reflectionExtender';
import IEnumerator from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerator';
import { outputVariables } from '../../utils/outputVariables';


export class math_Sum extends mfunction {

    public /* override */ get minParameters(): number {
        return 1;
    }

    public /* override */ get maxParameters(): number {
        return -1;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        resolved.resolved = true;
        let result: number = 0;
        for (const ex of parameters.toArray()) {
            let item: Object = ex.resolveGeneric<Object>(expressionData, resolved, getType(Object), running);
            if (resolved.resolved) {
                if ((item instanceof List)) {
                    let e: List<any> = (<List<any>>(item));
                    let en: IEnumerator<any> = e.getEnumerator();
                    while (en.moveNext()) {
                        result = (result + <number>typeMapper.toType1<number>(en.current as Object, getType(Number)));
                    }

                }
                else {
                    result = (result + ex.resolveGeneric<number>(expressionData, /* out */resolved, getType(Number), running));
                }

            }

        }

        resolved.resolved = true;
        return result;
    }
}
export class mathFunctions {
}


export class math_Min extends mfunction {

    public /* override */ get minParameters(): number {
        return 1;
    }

    public /* override */ get maxParameters(): number {
        return -1;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        resolved.resolved = true;
        let min: number | undefined = undefined;
        for (let ex of parameters.toArray()) {
            let item: Object = ex.resolveGeneric<Object>(expressionData, resolved, getType(Object), running);
            if (resolved.resolved) {
                if ((item instanceof List)) {
                    let e: List<any> = (<List<any>>(item));
                    let en: IEnumerator<any> = e.getEnumerator();
                    while (en.moveNext()) {
                        let r: number = typeMapper.toType<number>(en.current) as number;
                        if (min == undefined)
                            min = r;
                        if ((r <= (min as number))) {
                            min = r;
                        }

                    }

                }
                else {
                    let r: number = ex.resolveGeneric<number>(expressionData, resolved, getType(Number), running);
                    if (min == undefined)
                        min = r;
                    if ((r <= (min as number))) {
                        min = r;
                    }

                }

            }

        }

        resolved.resolved = true;
        return min as number;
    }
}

export class math_Max extends mfunction {

    public /* override */ get minParameters(): number {
        return 1;
    }

    public /* override */ get maxParameters(): number {
        return -1;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {
        resolved.resolved = true;
        let max: number = 0;
        for (let ex of parameters.toArray()) {
            let item: Object = ex.resolveGeneric<Object>(expressionData, /* out */resolved, getType(Object), running);
            if (resolved) {
                if (item instanceof List) {
                    let e: List<any> = (<List<any>>(item));
                    let en: IEnumerator<any> = e.getEnumerator();
                    while (en.moveNext()) {
                        let r: number = typeMapper.toType<number>(en.current) as number;
                        if ((r >= max)) {
                            max = r;
                        }

                    }

                }
                else {
                    let r: number = ex.resolveGeneric<number>(expressionData, /* out */resolved, getType(Number), running);
                    if ((r >= max)) {
                        max = r;
                    }

                }

            }

        }

        resolved.resolved = true;
        return max;
    }
}
abstract class binary<T1, T2, Result> extends mfunction {

    public /* override */ get minParameters(): number {
        return 1;
    }

    public /* override */ get maxParameters(): number {
        return 2;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {
        resolved.resolved = true;
        let result: Result | undefined;
        let p1: T1 | undefined;
        let p2: T2 | undefined;
        if ((parameters.count > 1)) {
            p2 = parameters.get(1).resolve(expressionData, /* out */resolved, /* running:*/ running) as T2;
        }

        if (resolved) {
            p1 = parameters.get(0).resolve(expressionData, /* out */resolved, /* running:*/ running) as T1;
        }

        if (resolved) {
            result = this.subInvoke(p1 as T1, p2 as T2);
        }

        resolved.resolved = true;
        return result;
    }

    protected abstract subInvoke(param1: T1, param2: T2): Result;
}

abstract class unary<T1, Result> extends mfunction {

    public /* override */ get minParameters(): number {
        return 1;
    }

    public /* override */ get maxParameters(): number {
        return 1;
    }

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object | undefined {
        resolved.resolved = true;
        let result: Result | undefined;
        let p1: T1 | undefined;
        if (resolved) {
            p1 = parameters.get(0).resolve(expressionData, /* out */resolved, /* running:*/ running) as T1;
        }

        if (resolved) {
            result = this.subInvoke(p1 as T1);
        }

        resolved.resolved = true;
        return result;
    }

    protected /* internal */ abstract subInvoke(param1: T1): Result;
}

export class math_Round extends binary<number, number, number> {

    protected subInvoke(param1: number, param2: number): number {
        if (param2 == undefined || param2 == 0)
            return Math.round(param1);
        else {
            let pow = Math.pow(10, param2);
            return Math.round(param1 * pow) / pow;
        }
    }
}

export class math_Pow extends binary<number, number, number> {

    protected subInvoke(param1: number, param2: number): number {
        return Math.pow(param1, param2);
    }
}

export class math_Log extends binary<number, number, number> {

    protected subInvoke(param1: number, param2: number): number {
        if (param2 == 0)
            return Math.log(param1);
        else
            return Math.log(param1);//, param2);
    }
}

export class math_Atan2 extends binary<number, number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number, param2: number): number {
        return Math.atan2(param1, param2);
    }
}

export class math_Abs extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.abs(param1);
    }
}

export class math_Sqrt extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.sqrt(param1);
    }
}

export class math_Asin extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.asin(param1);
    }
}

export class math_Acos extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.acos(param1);
    }
}

export class math_Atan extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.atan(param1);
    }
}

export class math_Sin extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.sin(param1);
    }
}

export class math_Cos extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.cos(param1);
    }
}

export class math_Tan extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.tan(param1);
    }
}

export class math_Sinh extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.sinh(param1);
    }
}

export class math_Cosh extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.cosh(param1);
    }
}

export class math_Tanh extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.tanh(param1);
    }
}

export class math_Exp extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        return Math.exp(param1);
    }
}

export class math_Pi extends unary<number, number> {

    protected /* internal */ /* override */ subInvoke(param1: number): number {
        if (param1 == 0)
            return Math.PI;
        else {
            let pow = Math.pow(10, param1);

            return Math.round(Math.PI * pow) / pow;
        }
    }
}